package net.java.dev.fwutil;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.lang.reflect.Constructor;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.WindowConstants;

/** A window that is or has a root/parent window with possibly child windows. */ 
public class MFrame extends JFrame implements ParentWindow {
	
	private static final long serialVersionUID = -1584624453975627603L;
	protected ParentWindow parentWindow;
	protected final ArrayList<MFrame> childWindows = new ArrayList<MFrame>();

	public MFrame(ParentWindow parentWindow, String windowTitle) {
		
		super(windowTitle);
		this.parentWindow = parentWindow;
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(final WindowEvent e) {
				dispose();
			}
		});
	}
	
	public static void openWindow(final Class<?> clazz, final ParentWindow parentWindow, 
			final String windowTitle) {
		
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					Constructor<?> con = clazz.getConstructor(ParentWindow.class, String.class);
					con.newInstance(parentWindow, windowTitle);
				} catch (Exception e) {
					Log.logError("Failed to open window " + windowTitle);
					Log.logError(e);
				}
			}
		});
	}
	
	public void addChildWindow(MFrame childWindow) {
		
		if (!childWindows.contains(childWindow))
			childWindows.add(childWindow);
	}
	
	public boolean removeChildWindow(MFrame childWindow) {
		return childWindows.remove(childWindow);
	}
	
	protected void closeChildWindows() {
		
		while (childWindows.size() > 0)
			childWindows.get(0).dispose();
	}
	
	protected void display(final Component owner) {
		
		pack();
		WindowDimension.applyWindowDimension(getTitle(), this, owner);
		setVisible(true);
		if (parentWindow != null)
			parentWindow.addChildWindow(this);
	}
	
	/** If overriden, call super() at the end. 
	 * The final statement in this method releases all UI-resources.*/ 
	public void dispose() {
		
		closeChildWindows();
		WindowDimension.setWindowDimension(getTitle(), this);
		if (parentWindow != null)
			parentWindow.removeChildWindow(this);
		setVisible(false);
		super.dispose();
	}
	
	public void makeVisible() {

		if (!isVisible())
			setVisible(true);
		toFront();
	}
	
	public void setWaitCursor() {
		getContentPane().setCursor(new Cursor(Cursor.WAIT_CURSOR));
	}
	
	public void setNormalCursor() {
		getContentPane().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}
}
