package net.java.dev.fwutil;

import java.awt.Component;

import javax.swing.JOptionPane;


/**
 * A singleton class containing convenience methods for simple UI dialogs.
 * @author Fred
 *
 */
public class UIMessages {
	
	@SuppressWarnings("unused")
	private static UIMessages thisInstance = new UIMessages();
	
	private UIMessages() {
		super();
	}

	/** Pops up a Yes/No question. Returns true if Yes was clicked. */
	public static boolean askYesNo(Component parent, String title, Object message) {
		
		if (title == null)
			title = "Question";
		if (JOptionPane.showConfirmDialog(parent, message, title, JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
			return true;
		return false;
	}
	
	/** Pops up an information message. */
	public static void showMessage(Component parent, String title, Object message) {
		
		if (title == null)
			title = "Information";
		JOptionPane.showMessageDialog(parent, message, title, JOptionPane.PLAIN_MESSAGE);
	}
	
	/** Pops up an error message and logs it. */
	public static void showErrorMessage(Component parent, String title, Object message) {
		
		if (title == null)
			title = "Error";
		if (message instanceof Throwable)
			Log.logError((Throwable)message);
		else
			Log.logError(title + System.getProperty("line.separator") + message.toString());
		JOptionPane.showMessageDialog(parent, message, title, JOptionPane.ERROR_MESSAGE);
	}
	
	/** Pops up an error message but does not log it (e.g. "value for field x is mandatory."-message */
	public static void showUserErrorMessage(Component parent, String title, Object message) {
		
		if (title == null)
			title = "Error";
		JOptionPane.showMessageDialog(parent, message, title, JOptionPane.ERROR_MESSAGE);
	}

	/** Asks for a String and returns the user's input. */
	public static String askLabel(Component parent, String title, String question) {
		
		if (title == null)
			title = "Question";
		return JOptionPane.showInputDialog(parent, question, title, 
				JOptionPane.QUESTION_MESSAGE);
	}
}
