package net.java.dev.fwutil.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Desktop;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.UIMessages;

/** Base File menu with Open log file, Delete settings on exit,
 * restart and exit.
 * @author Fred
 *
 */
public class BMFile extends JMenu implements ActionListener {

	private static final long serialVersionUID = 6110242369587664493L;
	
	protected IMainWindow mainWindow;
	protected JCheckBoxMenuItem mdelsettings;
	
	public BMFile(IMainWindow mainWindow) {
		super(Labels.get("File"));
		this.mainWindow = mainWindow;
		setMnemonic(KeyEvent.VK_F);
		addMenuItems();
	}
	
	/** Override to add your own menu-items. */
	public void addMenuItems() {
		
		JMenuItem m;
		m = new JMenuItem(Labels.get(BMLabel.OpenLog));
		m.setMnemonic(KeyEvent.VK_L);
		m.setActionCommand(BMLabel.OpenLog);
		m.addActionListener(this);
		add(m);
		mdelsettings = new JCheckBoxMenuItem(Labels.get(BMLabel.DeleteSettings));
		mdelsettings.setSelected(false);
		m = mdelsettings;
		m.setMnemonic(KeyEvent.VK_D);
		add(m);
		addSeparator();

		if (mainWindow.getMainRestart() != null) {
			m = new JMenuItem(Labels.get(BMLabel.Restart));
			m.setMnemonic(KeyEvent.VK_R);
			m.setActionCommand(BMLabel.Restart);
			m.addActionListener(this);
			add(m);
		}
		m = new JMenuItem(Labels.get(BMLabel.ExitApp));
		m.setMnemonic(KeyEvent.VK_X);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, KeyEvent.ALT_DOWN_MASK));
		m.setActionCommand(BMLabel.ExitApp);
		m.addActionListener(this);
		add(m);
	}
	
	public boolean deleteSettingsSelected() {
		return mdelsettings.isSelected();
	}

	/** Override to catch your own action events from menu items. */
	public void actionPerformed(ActionEvent ae) {
		
		final String ac = ae.getActionCommand();
		if (BMLabel.Restart == ac || BMLabel.ExitApp == ac) {
			mainWindow.exitApp(BMLabel.Restart == ac);
		} else if (BMLabel.OpenLog == ac) {
			if (Desktop.isDesktopSupported()) {
				try {
					Desktop.getDesktop().open(new File(Log.getLogFileName()));
				} catch (Exception ex) {
					UIMessages.showErrorMessage(null, BMLabel.OpenLog, Labels.get("OpenFileFailed")
							+ ex);
				}
			} else {
				UIMessages.showErrorMessage(null, BMLabel.OpenLog, Labels.get("JavaDesktopNA"));
			}
		}
	}
	
}
