package net.java.dev.fwutil.gui;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import net.java.dev.fwutil.Labels;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HBox;
import net.java.dev.hvlayout.VBox;

/** Basic status bar that allows selecting and copying text. */
public class BMainStatusBar extends VBox implements ActionListener, ClipboardOwner {
	
	private static final long serialVersionUID = -6958152772819584562L;
	
	public static final String CopyAction = "Copy";

	protected JTextField text;
	protected JPopupMenu popupMenu;
	
	public BMainStatusBar() {
		super();
		HBox hb = new HBox();
		text = FormUI.getTextField("", 0);
		text.setEnabled(true);
		text.setEditable(false);
		hb.add(text);
		
		KeyStroke controlC = KeyStroke.getKeyStroke("control C");
		JMenuItem mi = new JMenuItem(Labels.get(CopyAction));
		mi.setActionCommand(CopyAction);
		mi.setAccelerator(controlC);
		mi.setMnemonic(KeyEvent.VK_C);
		mi.addActionListener(this);
		popupMenu = new JPopupMenu();
		popupMenu.add(mi);
		text.add(popupMenu);
		PopupListener popupListener = new PopupListener(); 
		text.addMouseListener(popupListener);

		add(hb);
	}
	
	public void setText(final String s) {
		text.setText(s);
	}
	
	class PopupListener extends MouseAdapter {

		public void mousePressed(MouseEvent e) {
			showPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			showPopup(e);
		}

		private void showPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popupMenu.show(e.getComponent(), e.getX(), e.getY());
				e.consume();
			}
		}
	}

	public void actionPerformed(ActionEvent ae) {
		
		if (ae.getActionCommand() == CopyAction) {
			Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
			clipboard.setContents(new StringSelection(text.getText()), this);
		}
	}

	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		// do nothing
	}

}
