package net.java.dev.fwutil.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryUsage;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JButton;
import javax.swing.JLabel;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.MFrame;
import net.java.dev.fwutil.ParentWindow;
import net.java.dev.fwutil.TableCellNumberRenderer;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HBox;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VBox;

public class BMemInfo extends MFrame implements ActionListener {
	
	private static final long serialVersionUID = 2188497805713761360L;

	public static final long updateDelay = 1000;
	
	public static String windowIconFile = null;

	private JLabel memfree;
	private JLabel memallocated;
	private JLabel memavail;
	private JButton bfreeMem;
	private MemoryMXBean memorymbean;
	private boolean disposed = false;
	
	private static BMemInfo instance = null;
	private static Timer secondsTimer = new Timer();
	
	/** Shows the Memory-info frame and sets it as a child of parentWindow.
	 * Component is used for 
	 * @param parentWindow not null
	 * @param owner can be null
	 */
	public static void show(final ParentWindow parentWindow, final Component owner) {
		
		if (instance == null || instance.isDisposed()) {
			javax.swing.SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					instance = new BMemInfo(parentWindow, owner);
				}
			});
		} else {
			instance.setVisible(true);
			instance.toFront();
		}
	}
	
	/** No need to use this method: window gets disposed with parent-window 
	 * and can be closed by user. 
	 */
	public static void close() {
		
		if (instance != null && !instance.isDisposed()) {
			instance.dispose();
			instance = null;
		}
	}

	private boolean isDisposed() {
		return disposed;
	}

	private BMemInfo(ParentWindow parentWindow, Component owner) {
		super(parentWindow, Labels.get("MemoryInfoTitle"));
		
		VBox mainPane = new VBox(null);
		HBox lb = new HBox();
		lb.add(FormUI.getLabel(Labels.get("FreeMemory"), 0, true));
		memfree = FormUI.getLabel("", HVProps.defaultLineWidth, true);
		lb.add(memfree);
		mainPane.add(lb);
		lb = new HBox();
		lb.add(FormUI.getLabel(Labels.get("UsedMemory"), 0, true));
		memallocated = FormUI.getLabel("", HVProps.defaultLineWidth, true);
		lb.add(memallocated);
		mainPane.add(lb);
		lb = new HBox();
		lb.add(FormUI.getLabel(Labels.get("AvailMemory"), 0, true));
		memavail = FormUI.getLabel("", HVProps.defaultLineWidth, true);
		lb.add(memavail);
		mainPane.add(lb);
		lb = new HBox();
		bfreeMem = FormUI.getWideButton(Labels.get("FreeMemoryNow"));
		bfreeMem.addActionListener(this);
		lb.add(bfreeMem);
		mainPane.add(lb);
		mainPane.add(FormUI.getCanvasLine());
		setContentPane(mainPane);
		memorymbean = ManagementFactory.getMemoryMXBean();
		//mempoolsmbeans = ManagementFactory.getMemoryPoolMXBeans();
		//gcmbeans = ManagementFactory.getGarbageCollectorMXBeans();
		if (windowIconFile != null) BMainWindow.loadWindowIcon(this, windowIconFile);
		display(owner);
		showMemInfo();
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == bfreeMem) {
			for (int r = 0; r < 4; ++ r) System.gc();
		}
	}
	
	/**
	 * Parts copied from
	 * http://blog.codebeach.com/2008/02/determine-available-memory-in-java.html
	 * but more from
	 * http://www.informit.com/guides/content.aspx?g=java&seqNum=249
	 *
	 */
	public void showMemInfo() {

		if (!disposed) {
			MemoryUsage heapmem = memorymbean.getHeapMemoryUsage();
			MemoryUsage nonheapmem = memorymbean.getNonHeapMemoryUsage();
			long totalAllocated = heapmem.getCommitted() + nonheapmem.getCommitted();
			long totalAvailable = heapmem.getMax() + nonheapmem.getMax();
			long totalFree = totalAllocated - (heapmem.getUsed() + nonheapmem.getUsed());  
			memfree.setText(TableCellNumberRenderer.numberFormat.format(
					totalFree / 1024) + " kB");
			memallocated.setText(TableCellNumberRenderer.numberFormat.format(
					totalAllocated / 1024) + " kB");
			memavail.setText(TableCellNumberRenderer.numberFormat.format(
					(totalAvailable - totalAllocated) / 1024) + " kB");
			secondsTimer.schedule(new memInfoUpdateTask(), updateDelay);
		}
	}
	
	class memInfoUpdateTask extends TimerTask {
		public void run() {
			showMemInfo();
		}
	}
	
	public void dispose() {

		disposed = true;
		secondsTimer.cancel();
		super.dispose();
		bfreeMem = null;
	}

}
