/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */
package net.java.dev.hvlayout;

import java.awt.Font;
import java.awt.event.MouseEvent;

import javax.swing.Icon;
import javax.swing.JLabel;


/** 
 * A JLabel that shows a tooltip when the label-text is truncated.
 */ 
public class HVLabel extends JLabel {
	
	private static final long serialVersionUID = 5815324549890565898L;

	public HVLabel() {
        this(null, null, LEADING);
	}
	
	public HVLabel(String text) {
        this(text, null, LEADING);
	}

	public HVLabel(String text, Font font) {
        this(text, null, LEADING);
		if (font != null)
			setFont(font);
	}

	public HVLabel(Icon image) {
        this(null, image, CENTER);
	}

	public HVLabel(Icon image, int horizontalAlignment) {
        this(null, image, horizontalAlignment);
	}

	public HVLabel(String text, int horizontalAlignment) {
        this(text, null, horizontalAlignment);
	}
	
	public HVLabel(String text, Icon image, int horizontalAlignment) {
		super(text, image, horizontalAlignment);
		setIconTextGap(2);
		setToolTipText(" ");
		setRequestFocusEnabled(false);
	}
	
	/**
	 * Shows a tooltip with the full text of the label
	 * when the text in the label is truncated (i.e. when the three
	 * dots appear at the end of the text a.k.a. ellipsis) and no tooltip-text is set.
	 * Copied and modified from
	 * http://forum.java.sun.com/thread.jspa?threadID=675342&messageID=3943542
	 */
	public String getToolTipText(MouseEvent e) {
		
		String text = super.getToolTipText();
		if (!text.equals(" "))
				return text;
		text = getText();
		int textWidth = getFontMetrics(getFont()).stringWidth(text);
		int iconWidth = (getIcon() == null ? 0 : getIcon().getIconWidth() + getIconTextGap());
		int textSpace = getWidth() - iconWidth 
			- getInsets().left - getInsets().right;
		return (textWidth > textSpace ? text : null);
	}
}
