/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */
package net.java.dev.hvlayout;

import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableRowSorter;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.table.AbstractTableModel;

/**
 * A table sorting demo (can be executed) that shows a table with some data
 * that can be sorted on all columns.
 * The sorting is enabled via HVTableHeader which uses
 * HVTableHeaderButton. The actual sorting is done by
 * Swing with TableRowSorter. HVTableHeader is needed to
 * be able to use the sorting available via TableRowSorter.
 * HVTableHeaderButton is needed to show on screen what the sorting settings are
 * (this includes sort order and sort up or down).
 * HVTableHeaderButton generates icons shown in the table header
 * to indicate the sort order (by showing a number) and type 
 * (by showing an up- or down-arrow).
 *  
 * @author Fred
 *
 */
public class TableHeaderDemo {
	
	/** Needed to position the window at middle of the screen. */
	public static final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	/** The amount of columns in tdata. */
	public static final int columns = 4;
	/** The table data that is used by the table model and displayed in the table. */
	public static final Boolean[][] tdata = 
		{{true, true, true, true},
		{true, true, true, false},
		{true, true, false, false},
		{true, false, false, false},
		{false, false, false, false},
		{false, true, true, true},
		{false, false, true, true},
		{false, false, false, true},
		{false, true, false, true},
		{true, false, true, false},
		{false, true, true, false},
		{true, false, false, true}};
	
	private JFrame thdframe;
	/**
	 * Shows a window with a table that can be sorted.
	 */
	public TableHeaderDemo() {
		
		thdframe = new JFrame("TableHeader Demo");
		thdframe.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		
		// The table-model that uses tdata
		THDModel thdmodel = new THDModel();
		JTable thdtable = new JTable(thdmodel);
		
		// The following two statements are needed to allow sorting
		thdtable.setRowSorter(new TableRowSorter<THDModel>(thdmodel));
		thdtable.setTableHeader(new HVTableHeader(thdtable.getColumnModel()));
		
		// Build the screen
		
		JScrollPane thdscroller = HVProps.getScroller(thdtable, 0, 15, false);
		VBox mainPane = new VBox(null);
		mainPane.add(FormUI.getLabel("Use left click to set the first sort column.", 0));
		mainPane.add(FormUI.getLabel("Use right click to add another sort column,", 0));
		mainPane.add(FormUI.getLabel("or use right click to reverse sort order.", 0));
		mainPane.add(thdscroller);
		thdframe.setContentPane(mainPane);
		thdframe.pack();
		// Position at the middle of the screen.
		thdframe.setLocation((int)((screenSize.getWidth() - thdframe.getSize().getWidth()) / 2),
				  (int)((screenSize.getHeight() - thdframe.getSize().getHeight()) / 2));
		thdframe.setVisible(true);
	}
	
	public JFrame getFrame() {
		return thdframe; 
	}
	
	/** Starts TableHeaderDemo() */
	public static void main(String[] args) {
		
		try {
			UIManager.setLookAndFeel(
					UIManager.getSystemLookAndFeelClassName());
		} catch (Exception unused) { }
		// Needed to ensure the window is shown.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new TableHeaderDemo();
			}
		});
	}
	
	/** A simple table model that uses tdata. */
	class THDModel extends AbstractTableModel {

		private static final long serialVersionUID = -2647342849785167574L;

		public int getColumnCount() {
			return columns;
		}

		public int getRowCount() {
			return tdata.length;
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			return tdata[rowIndex][columnIndex];
		}
		
		public Class<?> getColumnClass(int columnIndex) {
			return Boolean.class;
		}
	}
}
