package net.java.dev.fwutil.pf;

import java.io.File;
import java.io.IOException;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.Util;

public final class OpenFile {
	
	public static final String OpenShellCommand = "openShellCommand";

	private OpenFile() {super();}
	
	public static File getCanonical(File f) {
		
		if (f == null) return null;
		try {
			f = f.getCanonicalFile();
		} catch (IOException ioe) {
			Log.logError("Could not get canonical file for " + f.getPath() + Log.exception2String(ioe));
		}
		return f;
	}

	public static void openFile(final File f) {
		openFile(f, false);
	}
	
	public static void editFile(File f) {
		openFile(f, true);
	}
	
	public static void openShell(File f) {

		if (f.isDirectory())
			Execute(OpenShellCommand, f.getAbsolutePath());
		else 
			Execute(OpenShellCommand, f.getParent());
	}
	
	public static void openFile(File f, final boolean openForEdit) {
	
		if (f == null || !f.exists()) return;
		f = getCanonical(f);
		if (java.awt.Desktop.isDesktopSupported()) {
			try {
				Log.log("Opening file (for edit <" + openForEdit + ">): " + f.getPath());
				if (openForEdit)
					java.awt.Desktop.getDesktop().edit(f);
				else
					java.awt.Desktop.getDesktop().open(f);
			} catch (IOException ioe) {
				Log.logError("Could not open file " + f.getPath() + Util.newline + Log.exception2String(ioe));
			}
		} else {
			if (DiskInfo.windowsOS) {
				Util.Execute("rundll32 url.dll,FileProtocolHandler \"" + f.getPath() + "\"");
			}
		}
	}

	public static void customCmd1(File f) {

		if (f == null || !f.exists()) return;
		f = getCanonical(f);
		String command = Labels.get("Custom1File") + " \"" + f.getPath() + "\"";
		if (command.toUpperCase().contains("RUNDLL32")) 
			command = Labels.get("Custom1File") + " " + f.getPath();
		Execute(command, Labels.get("Custom1Dir"));
	}

	public static void customCmd2(File f) {
		
		if (f == null || !f.exists()) return;
		f = getCanonical(f);
		String command = "\"" + Labels.get("Custom2File") + "\"  \""  + f.getPath() +
			(f.isDirectory() ? File.separator + "\" " : "\" ");
		if (command.toUpperCase().contains("RUNDLL32")) 
			command = Labels.get("Custom1File") + " " + f.getPath();
		Execute(command, Labels.get("Custom2Dir"));
	}
	
	/** Executes the command in the OS. */
	public static Process Execute(final String command, String dirName) {
		
		File dir = new File(dirName);
		if (!dir.exists() || !dir.isDirectory()) {
			Log.logError("Invalid directory: " + dirName);
			return null;
		}
		Log.log("Executing [" + command + "] in [" + dirName + "]");
		Process p = null;
		Runtime r = Runtime.getRuntime();
		dir = getCanonical(dir);
		if (DiskInfo.windowsOS) { 
			try {
				if (Util.osName.indexOf("windows 9")>-1) {  // Win95-98
					if (OpenShellCommand.equals(command)) {
						p = r.exec("command", null, dir);
					} else {
						p = r.exec("command /c " + command, null, dir);
					}
				} else { // Winnt, 2000, XP
					if (OpenShellCommand.equals(command)) {
						//Log.log("Executing opening shell on WINDOWS");
						p = r.exec("cmd /c start", null, dir);
					} else {
						p = r.exec("cmd /c start \"\" " + command, null, dir);
					}
				}
			} catch (IOException ioe) {
				Log.logError(ioe);
				Util.closeProcess(p);
			}
		} else { // assume Unix
			try {
				if (OpenShellCommand.equals(command)) {
					p = r.exec(System.getenv("TERM"), null, dir);
				} else {
					p = r.exec("/bin/bash -c " + command, null, dir);
				}
			} catch (IOException ioe) {
				Log.logError(ioe);
				Util.closeProcess(p);
			}
		}
		return p;
	}
}
