package net.java.dev.fwutil.pf.ui;

//import java.awt.Cursor;
import java.awt.AWTKeyStroke;
import java.awt.KeyboardFocusManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.Util;
import net.java.dev.fwutil.pf.DiskInfo;
import net.java.dev.fwutil.pf.FileInfo;
import net.java.dev.fwutil.pf.OpenFile;
import net.java.dev.fwutil.pf.Params;
import net.java.dev.fwutil.pf.ParentDir;
import net.java.dev.fwutil.pf.ui.table.FileTable;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HVProps;

public class BrowseLocation implements KeyListener, ActionListener, DocumentListener, MouseListener {

	private boolean showHiddenFiles = Params.showHiddenFiles();
	public boolean showDirectoriesOnly = false;
	private String lastPathKey;

	private DiskInfo diskInfo;
	private JTextField textInput;
	private FileTable fileTable;
	private String lastUserPath;
	private boolean ignoreRemoveEvent;
	private List<FileInfo> lastFileList;
	private List<FileInfo> lastFilteredFileList;

	public BrowseLocation(DiskInfo diskInfo) {
		super();
		this.diskInfo = diskInfo;

		textInput = FormUI.getTextField(Util.emptyString, (int)(HVProps.defaultLineWidth * 3.5));
		/* Most keyactions for textInput are registered by BrowsePanel. */
		
		/* It is not possible to listen to TAB using the InputMap. 
		 * Instead, remove the tab-listener (as done below) ... */
		textInput.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, Collections.<AWTKeyStroke>emptySet());
		/* .. and at our own key-listener. */
		textInput.addKeyListener(this);
		/* Listen to enter-key-pressed. */
		textInput.addActionListener(this);
		textInput.getDocument().addDocumentListener(this);

		lastPathKey = lastUserPath = DiskInfo.RootDir;
		ignoreRemoveEvent = false;
		lastFileList = lastFilteredFileList = null;
	}
	
	public JTextField getTextField() {
		return textInput;
	}
	
	public String getLastPathKey() {
		return lastPathKey;
	}
	
	public void setFileTable(FileTable fileTable) {
		
		if (this.fileTable != null) this.fileTable.removeMouseListener(this);
		this.fileTable = fileTable;
		fileTable.addMouseListener(this);
	}
	
	public void setShowHiddenFiles(boolean showHiddenFiles) {
		this.showHiddenFiles = showHiddenFiles;
		refreshLastDir();
	}
	
	public void refreshLastDir() {

		Log.log("Refreshing directory contents for " + lastPathKey);
		diskInfo.refreshDirectoryContents(lastPathKey);
		setPathKey(lastPathKey, true);
	}
	
	public void selectParentDir() {
		
		if (lastPathKey != null && lastPathKey.indexOf(File.separator) > -1) {
			File f = new File(lastPathKey);
			if (f.getParent() == null) {
				focusRootLocation();
			} else {
				try {
					String filePath = DiskInfo.endWithSeparator(f.getParentFile().getCanonicalPath());
					setPathKey(filePath, false);
					setNewPathKey(filePath);
				} catch (Exception e) {
					Log.logWarning("Failed to get parent directory: " + e);
				}
			}
		}
	}
	
	public void focusLocationText() {

		textInput.requestFocus();
		textInput.setSelectionStart(0);
		textInput.setSelectionEnd(textInput.getText().length());
	}
	
	public void focusRootLocation() {
		
		setPathKey(DiskInfo.RootDir, false);
		setNewPathKey(DiskInfo.RootDir);
		textInput.requestFocus();
	}

	public void setBookMarkLocation(String pathKey) {
		
		setPathKey(pathKey, false);
		setNewPathKey(pathKey);
	}

	public void openFileInSelectedTableRow() {
		
		if (fileTable.getSelectedRow() > -1) {
			File f = new File(lastPathKey + fileTable.getFileInfo(fileTable.getSelectedRow()).name);
			if (f.exists() && f.isFile()) {
				OpenFile.openFile(f, false);
			} else {
				setFile(fileTable.getFileInfo(fileTable.getSelectedRow()));
				ignoreRemoveEvent = false; // For Enter-event the text is already set.
			}
		}
	}

	public void actionPerformed(ActionEvent ae) {
		openFileInSelectedTableRow();
	}

	public void mouseClicked(MouseEvent e) {

		if (e.getClickCount() % 2 == 0 && //even clicks, 2,4,6,etc
				SwingUtilities.isLeftMouseButton(e)) {
			actionPerformed(new ActionEvent(this, 0, "DoubleMouseClick"));
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	
	public void keyPressed(final KeyEvent e) {
		
		if (e.getKeyCode() == KeyEvent.VK_TAB && e.getModifiers() == 0) {
			//Log.log("Tab pressed");
			final String userPath = textInput.getText();
			if (userPath == null || userPath.length() == 0) {
				setPathKey(DiskInfo.RootDir, false);
				return;
			}
			File f = new File(userPath);
			if (f.exists() && lastUserPath.equals(userPath)
					&& !(lastFilteredFileList.size() <= 2)) {
				KeyboardFocusManager.getCurrentKeyboardFocusManager().focusNextComponent();
			} else {
				processUserPath(true);
			}
		}
	}
	public void keyReleased(final KeyEvent e) {}
	public void keyTyped(final KeyEvent e) {}

	public void changedUpdate(DocumentEvent e) {
		//Log.log("Document changed event: " + e);
		processUserPath(false);
	}
	public void insertUpdate(DocumentEvent e) {
		//Log.log("Document insert event: " + e);
		processUserPath(false);
	}
	public void removeUpdate(DocumentEvent e) {
		
		// When text is set in the textfield, 
		// a remove event is fired. But there is no need to act on it
		// since an insert events will be fired soon afterwards.
		// Must not be set when text was set by Enter-event.
		if (ignoreRemoveEvent) {
			ignoreRemoveEvent = false;
			return;
		}
		// Alternative to ignoreRemoveEvent (does not work very well)
		// if ((textInput.getText() == null || textInput.getText().length() == 0)
		//		&& (e.getLength() > 0)) return;
		//Log.log("Document remove event: " + e);
		processUserPath(false);
	}

	public void processUserPath(final boolean pathComplete) {
		
		//final boolean pathComplete = true;
		final String userPath = textInput.getText();
		if (lastUserPath.equals(userPath) && !pathComplete) return;
		Log.log("Processing path: " + userPath + " pathComplete " + pathComplete);
		// Use invokeLater to prevent "writeLock" error on document.
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if (userPath == null || userPath.length() == 0 || userPath.equals(File.separator)) {
					setPathKey(DiskInfo.RootDir, false);
					if (!DiskInfo.windowsOS)
						lastUserPath = File.separator;
					else 
						lastUserPath = Util.emptyString;
					updateText();
					return;
				}
				File f = new File(userPath);
				if (userPath.length() < lastUserPath.length()) {
					if (f.exists() && f.isDirectory()) {
						lastUserPath = userPath;
						setPathKey(DiskInfo.getPathKey(f), false);
						return;
					}
					if (userPath.length() < lastPathKey.length()) {
						if (f.getParent() == null) {
							setPathKey(DiskInfo.RootDir, false);
						} else {
							File fparent = new File(f.getParent());
							if (fparent.exists() && fparent.isDirectory()) {
								lastUserPath = userPath;
								setPathKey(DiskInfo.getPathKey(fparent), false);
								return;
							}
						}
					}
				}
				Log.log("LastPathKey: " + lastPathKey);
				String searchText = (userPath.equals(lastPathKey) ? "" : f.getName());
				if (lastPathKey == DiskInfo.RootDir) searchText = userPath;
				lastFilteredFileList = DiskInfo.filterFileList(lastFileList, searchText);
				if (lastFilteredFileList.size() > 0) {
					Log.log("Setting filtered list .. ");
					fileTable.getModel().setFiles(lastFilteredFileList);
					fileTable.setRowSelectionInterval(0, 0);
				} else {
					Log.log("Got empty file list!");
				}
				if (pathComplete) {
					if ( (lastFilteredFileList.size() == 1 || 
							(lastFilteredFileList.size() == 2 && ParentDir.ParentDirPath.equals(lastFilteredFileList.get(0).name)))) {
						int index = (lastFilteredFileList.size() == 2 ? 1 : 0);
						setFile(lastFilteredFileList.get(index));
					} else {
						final String prefix = DiskInfo.getPrefixName(lastFilteredFileList); 
						Log.log("Got prefix: " + prefix);
						if (prefix == null) return;
						lastUserPath = lastPathKey + prefix;
						updateText();
					}
				} else {
					if (userPath.length() > lastUserPath.length()) {
						lastUserPath = userPath;
						fileTable.getModel().setFiles(lastFilteredFileList);
						fileTable.setRowSelectionInterval(0, 0);
					}
				}
			}
			private void updateText() {
				ignoreRemoveEvent = true;
				textInput.setText(lastUserPath);
				textInput.setCaretPosition(textInput.getDocument().getLength());
			}
		});
	}
	
	public void setFile(final FileInfo fileInfo) {
		
		if (fileInfo == null) return;
		String filePath = lastPathKey + fileInfo.name; 
		File f = new File(filePath);
		if (f.exists() && f.isDirectory()) {
			String parentDir = f.getParent();
			if (parentDir != null && fileInfo.name.equals(ParentDir.ParentDirPath)) {
				parentDir = new File(parentDir).getParent();
			}
			if (parentDir == null && fileInfo.name.equals(ParentDir.ParentDirPath)) {
				filePath = DiskInfo.RootDir;
				setPathKey(filePath, false);
			} else {
				try {
					filePath = DiskInfo.endWithSeparator(f.getCanonicalPath());
					setPathKey(filePath, false);
				} catch (IOException ieo) {
					Log.logError("Problem getting canonical path for " + filePath + "\n" + Log.exception2String(ieo));
				}
			}
		}
		setNewPathKey(filePath);
	}
	
	public void setPathKey(String pathKey, final boolean refresh) {
		
		if (pathKey == null) pathKey = DiskInfo.RootDir;
		if ( (refresh || !lastPathKey.equals(pathKey)) || lastFileList == null) {
			lastPathKey = pathKey;
			lastFileList = diskInfo.getDirectoryContents(lastPathKey, showDirectoriesOnly, showHiddenFiles);
			lastFilteredFileList = lastFileList;
			if (lastPathKey == DiskInfo.RootDir) {
				lastFileList.add(0, DiskInfo.getUserHome());
			} else {
				lastFileList.add(0, new ParentDir());
			}
		}
		fileTable.getModel().setFiles(lastFileList);
		if (lastPathKey == DiskInfo.RootDir || fileTable.getModel().getRowCount() == 1) {
			fileTable.setSelectedRow(0);
		} else if (fileTable.getModel().getRowCount() > 1) {
			fileTable.setRowSelectionInterval(1, 1);
		}
	}

	/** Call this method after calling setPathKey(newPathKey, false) */ 
	public void setNewPathKey(final String pathKey) {
		
		lastUserPath = pathKey;
		Log.log("Setting text to " + pathKey);
		ignoreRemoveEvent = true;
		textInput.setText(pathKey);
		textInput.setCaretPosition(textInput.getDocument().getLength());
	}
	
	
	/** Returns true when both values are equal, including null. */
	public static final boolean equalValues(final Object v1, final Object v2) {
		
		return (( (v1 == null && v2 == null) || (v1 != null && v1.equals(v2)) ) ? true : false);
	}

}
