package net.java.dev.fwutil.pf.ui;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import net.java.dev.fwutil.pf.DiskInfo;
import net.java.dev.fwutil.pf.ui.table.FileTable;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VBox;

public class BrowsePanel {

	private static final long serialVersionUID = 4056989303024786183L;
	
	public static final String NextFile = "NextFile";
	public static final String PreviousFile = "PreviousFile";

	private VBox mainPanel;
	private FileDetailsPanel fileDetailsPanel;
	private JScrollPane mainScroller;

	private FileTable fileTable;
	private JScrollPane fileTableScroller;
	private BrowseLocation location;
	public JMenuItem showFileDetails = null;
	
	public BrowsePanel(DiskInfo diskInfo) {
		super();
		mainPanel = new VBox(null); // Sets default insets.
		location = new BrowseLocation(diskInfo);
		mainPanel.add(location.getTextField());
		fileTable = new FileTable(location, this.getClass().getName());
		location.setFileTable(fileTable);
		fileTableScroller = HVProps.getScroller(fileTable, 0, 20, false);
		mainPanel.add(fileTableScroller);
		fileDetailsPanel = new FileDetailsPanel(this); 
		//add(fileInfoView);
		setActions(location.getTextField());
		// Not a valid value for lastPathKey but setPathKey(null) will make it valid.
		location.setPathKey(DiskInfo.RootDir, false);
		mainScroller = HVProps.getScroller(mainPanel, false);
		mainScroller.setFocusable(false);
	}
	
	@SuppressWarnings({ "serial"})
	private void setActions(JTextField textInput) {
		
		textInput.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), "nextFile");
		textInput.getActionMap().put("nextFile", new AbstractAction(){
			public void actionPerformed(final ActionEvent ae) {
				//Log.log("Next file fired");
				if (fileTable.getSelectedRow()+1 < fileTable.getRowCount()) {
					fileTable.setRowSelectionInterval(fileTable.getSelectedRow()+1, fileTable.getSelectedRow()+1);
				}
			}});
		textInput.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), "previousFile");
		textInput.getActionMap().put("previousFile", new AbstractAction() {
			public void actionPerformed(final ActionEvent ae) {
				//Log.log("Previous file fired");
				if (fileTable.getSelectedRow() > 0) {
					fileTable.setRowSelectionInterval(fileTable.getSelectedRow()-1, fileTable.getSelectedRow()-1);
				}
			}});
		textInput.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN, 0), "pageDown");
		textInput.getActionMap().put("pageDown", new AbstractAction() {
			public void actionPerformed(final ActionEvent ae) {
				//Log.log("Page down fired");
				fileTable.scrollPageDown(fileTableScroller.getViewport());
			}});
		textInput.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP, 0), "pageUp");
		textInput.getActionMap().put("pageUp", new AbstractAction(){
			public void actionPerformed(final ActionEvent ae) {
				//Log.log("Page up fired");
				fileTable.scrollPageUp(fileTableScroller.getViewport());
			}});
		textInput.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_END, KeyEvent.CTRL_DOWN_MASK), "toBottom");
		textInput.getActionMap().put("toBottom", new AbstractAction(){
			public void actionPerformed(final ActionEvent ae) {
				//Log.log("Page up fired");
				fileTable.scrollToBottom();
			}});

		textInput.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_HOME, KeyEvent.CTRL_DOWN_MASK), "toTop");
		textInput.getActionMap().put("toTop", new AbstractAction() {
			public void actionPerformed(final ActionEvent ae) {
				//Log.log("Page down fired");
				fileTable.scrollToTop();
			}});
	}
	
	public JComponent getPanel() {
		return mainScroller;
	}
	
	public BrowseLocation getLocation() {
		return location;
	}
	
	public FileTable getTable() {
		return fileTable;
	}
	
	public void showFileDetails(boolean show) {

		if (showFileDetails.isSelected()) {
			mainPanel.add(fileDetailsPanel);
		} else {
			mainPanel.remove(fileDetailsPanel);
		}
		mainPanel.revalidate();
	}
	
	public void dispose() {
		fileTable.dispose();
	}
}
