package net.java.dev.fwutil.pf.ui;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.TableCellNumberRenderer;
import net.java.dev.fwutil.gui.BMainStatusBar;
import net.java.dev.fwutil.pf.FileInfo;
import net.java.dev.fwutil.pf.ParentDir;
import net.java.dev.fwutil.pf.ui.table.FileSizeCellRenderer;
import net.java.dev.fwutil.pf.ui.table.FileTableDataModel;

public class MainStatusBar extends BMainStatusBar implements TableModelListener {
	
	private static final long serialVersionUID = -6958152772819584562L;

	private FileTableDataModel dataModel;
	
	public MainStatusBar(final FileTableDataModel ftableModel) {
		super();
		dataModel = ftableModel;
		ftableModel.addTableModelListener(this);
	}

	public void tableChanged(TableModelEvent e) {

		if (e.getSource() == dataModel) {
			int fileCount = 0, dirCount = 0;
			long totalSize = 0;
			for (int i = 0; i < dataModel.getRowCount(); i++) {
				FileInfo fi = dataModel.getRow(i);
				if (fi.isDirectory) {
					if (!ParentDir.ParentDirPath.equals(fi.name)) {
						dirCount++;
					}
				} else {
					fileCount++;
					totalSize+=fi.size;
				}
			}
			if (dirCount == 0 && fileCount == 0) {
				text.setText(Labels.get("EmptyDir"));
				return;
			}
			StringBuilder info = new StringBuilder(TableCellNumberRenderer.numberFormat.format(dirCount) + " ");
			if (dirCount == 1) 
				info.append(Labels.get("Directory"));
			else
				info.append(Labels.get("directories"));
			info.append(" / ");
			info.append(TableCellNumberRenderer.numberFormat.format(fileCount));
			info.append(" ");
			if (fileCount == 1) 
				info.append(Labels.get("File"));
			else
				info.append(Labels.get("files"));
			info.append(" / ");
			info.append(FileSizeCellRenderer.getSize(totalSize));
			info.append(" (");
			info.append(TableCellNumberRenderer.numberFormat.format(totalSize));
			if (totalSize == 1) 
				info.append(Labels.get("Byte"));
			else
				info.append(" " + Labels.get("bytes") + ")");
			
			text.setText(info.toString());
		}
	}
}
