package net.java.dev.fwutil.pf.ui.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import net.java.dev.fwutil.FontSelector;
import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.LogUI;
import net.java.dev.fwutil.pf.Params;
import net.java.dev.fwutil.pf.ui.BrowsePanel;
import net.java.dev.fwutil.pf.ui.MainWindow;

public class MView extends JMenu implements ActionListener {

	private static final long serialVersionUID = 6110242369587664493L;
	
	private BrowsePanel browsePanel;
	private MainWindow mainWindow;
	private JMenuItem refresh;
	private JMenuItem parentDir;
	private JMenuItem sortName;
	private JMenuItem sortModified;
	private JMenuItem sortSize;
	private JMenuItem focusLocation;
	private JMenuItem focusRoot;
	private JMenuItem selectFont;
	private JCheckBoxMenuItem showFileDetails;
	private JCheckBoxMenuItem showHiddenFiles;

	MView(MainWindow mainWindow, BrowsePanel browsePanel) {
		super(Labels.get("View"));
		this.browsePanel = browsePanel;
		this.mainWindow = mainWindow;
		setMnemonic(KeyEvent.VK_V);
		JMenuItem m;
		refresh = new JMenuItem(Labels.get("Refresh"));
		m = refresh;
		m.setMnemonic(KeyEvent.VK_R);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		m.addActionListener(this);
		add(m);
		parentDir = new JMenuItem(Labels.get("SelectParentDir"));
		m = parentDir;
		m.setMnemonic(KeyEvent.VK_P);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_UP, KeyEvent.ALT_DOWN_MASK));
		m.addActionListener(this);
		add(m);
		
		focusLocation = new JMenuItem(Labels.get("FocusLocation"));
		m = focusLocation;
		m.setMnemonic(KeyEvent.VK_L);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, KeyEvent.CTRL_DOWN_MASK));
		m.addActionListener(this);
		add(m);
		focusRoot = new JMenuItem(Labels.get("FocusRootLocation"));
		m = focusRoot;
		m.setMnemonic(KeyEvent.VK_R);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_HOME, KeyEvent.ALT_DOWN_MASK));
		m.addActionListener(this);
		add(m);
		
		addSeparator();
		sortName = new JMenuItem(Labels.get("SortName"));
		m = sortName;
		m.setMnemonic(KeyEvent.VK_N);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0));
		m.setActionCommand("SortName");
		m.addActionListener(this);
		add(m);
		sortModified = new JMenuItem(Labels.get("SortModified"));
		m = sortModified;
		m.setMnemonic(KeyEvent.VK_M);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F7, 0));
		m.setActionCommand("SortModified");
		m.addActionListener(this);
		add(m);
		sortSize = new JMenuItem(Labels.get("SortSize"));
		m = sortSize;
		m.setMnemonic(KeyEvent.VK_S);
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F8, 0));
		m.setActionCommand("SortSize");
		m.addActionListener(this);
		add(m);
		
		addSeparator();
		showFileDetails = new JCheckBoxMenuItem(Labels.get("ShowFileDetails"), false);
		m = showFileDetails;
		m.setMnemonic(KeyEvent.VK_D);
		m.addActionListener(this);
		browsePanel.showFileDetails = m;
		m.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, KeyEvent.CTRL_DOWN_MASK));
		add(m);
		showHiddenFiles = new JCheckBoxMenuItem(Labels.get("ShowHiddenFiles"), Params.showHiddenFiles());
		m = showHiddenFiles;
		m.setMnemonic(KeyEvent.VK_H);
		m.addActionListener(this);
		add(m);
		
		addSeparator();
		selectFont = new JMenuItem(Labels.get("SelectFont"));
		m = selectFont;
		m.setMnemonic(KeyEvent.VK_F);
		m.setActionCommand("SelectFont");
		m.addActionListener(this);
		add(m);
	}
	
	public void actionPerformed(ActionEvent e) {
		
		if (e.getSource() == refresh) {
			browsePanel.getLocation().refreshLastDir();
		} else if (e.getSource() == parentDir) {
			browsePanel.getLocation().selectParentDir();
		} else if (e.getSource() == focusLocation) {
			browsePanel.getLocation().focusLocationText();
		} else if (e.getSource() == focusRoot) {
			browsePanel.getLocation().focusRootLocation();
		} else if (e.getSource() == sortName) {
			browsePanel.getTable().sortOnName();
		} else if (e.getSource() == sortModified) {
			browsePanel.getTable().sortOnModified();
		} else if (e.getSource() == sortSize) {
			browsePanel.getTable().sortOnSize();
		} else if (e.getSource() == showFileDetails) {
			browsePanel.showFileDetails(showFileDetails.isSelected());
		} else if (e.getSource() == showHiddenFiles) {
			Params.setShowHiddenFiles(showHiddenFiles.isSelected());
			browsePanel.getLocation().setShowHiddenFiles(showHiddenFiles.isSelected());
		} else if (e.getSource() == selectFont) {
			FontSelector.show(mainWindow);
		} else if (e.getSource() == focusRoot) {
			LogUI.makeVisible();
		}
	}
}
