package net.java.dev.fwutil.pf.ui.table;

import java.awt.Component;
import java.awt.ComponentOrientation;
import java.text.DecimalFormat;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.TableCellNumberRenderer;
import net.java.dev.fwutil.Util;

public class FileSizeCellRenderer extends DefaultTableCellRenderer {
	
	private static final long serialVersionUID = -2099594966409534423L;
	
	public static String[] sizes = { "B", "kB", "MB", "GB", "TB"};
	private static DecimalFormat nformatter = TableCellNumberRenderer.getNumberFormat();

	public FileSizeCellRenderer() {
		super();
		if (getComponentOrientation().isLeftToRight()) {
			applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		} else {
			applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		}
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, 
			boolean hasFocus, int row, int column) {
		
        if (isSelected || hasFocus) {
			setBackground(table.getSelectionBackground());
		} else {
			setBackground(table.getBackground());
		}
		setText(getSize((Long)value) + Util.space);
        return this;
    }

	public static String getSize(final long size) {
		
		String s;
		if (size <= 10240) {
			nformatter.setMaximumFractionDigits(0);
			s = nformatter.format(size) + " " + sizes[0];
		} else {
			int factor = 0;
			double dsize = (double)size;
			while (factor < sizes.length && dsize / 1024.0 > 1.0) {
				dsize = dsize / 1024.0;
				factor++;
			}
			factor = Math.min(sizes.length -1, factor);
			s = getSizeTextValue(dsize) + " " + sizes[factor];
		}
		return s;
	}
	
	public static String getSizeTextValue(final double value) {
		
		if (-10.0 < value && value < 10.0) {
			nformatter.setMaximumFractionDigits(2);
			nformatter.setMinimumFractionDigits(2);
		} else if (-100.0 < value && value < 100.0) {
			nformatter.setMaximumFractionDigits(1);
			nformatter.setMinimumFractionDigits(1);
		} else {
			nformatter.setMaximumFractionDigits(0);
		}
		String s = "";
		try {
			s = nformatter.format(value) + Util.space;
		} catch (Exception nan) {
			Log.logError("Invalid size number: " + value);
			s = Double.toString(value);
		}
		return s;
	}
}
