package nl.intercommit.mario.tools.jbossmon;

//import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.Session;
import javax.swing.table.AbstractTableModel;

import net.java.dev.fwutil.Log;
import nl.intercommit.mario.tools.JmsUtil;
import nl.intercommit.mario.tools.MarioContext;

public class MsgsTableModel extends AbstractTableModel {
	
	private static final long serialVersionUID = -5213737889510380104L;
	public MarioContext mc;
	private final QBrowser queueWindow;
	public String qName;
	private List<Message> messages;
	private RetrieveMessagesTask refreshTask = null;

	public MsgsTableModel(final MarioContext mc, final QBrowser smon, final String queueName) {

		super();
		this.mc = mc;
		queueWindow = smon;
		qName = queueName;
		//simpleDateFormat2 = new SimpleDateFormat("MM/dd HH:mm:ss yyyy");
		messages = new ArrayList<Message>();
	}

	public int getColumnCount() {
		return 7;
	}

	/**
	 * Return the column name of the JMS property.
	 * The following JMS properties give no value or an error when retrieved:
	 * - type
	 * - redelivered
	 * - replyto
	 * 
	 */
	public String getColumnName(final int column) {

		String columnName;
		switch (column) {
		case 0 : columnName = "JMS ID"; break;
		case 1 : columnName = "Description"; break;
		case 2 : columnName = "Timestamp"; break;
		case 3 : columnName = "Delivery Mode"; break;
		case 4 : columnName = "Expiration"; break;
		case 5 : columnName = "Priority"; break;
		case 6 : columnName = "Correlation ID"; break;
		default: columnName = "Unknown";
		};
		return columnName;
	}

	@Override
	public Class<?> getColumnClass(final int c) {

		Class<?> t;
		switch(c) {
		case 2 : t = Date.class; break; 
		case 3 : t = Integer.class; break;
		case 4 : t = Date.class; break;
		case 5 : t = Integer.class; break;
		default : t = String.class;
		};
		return t;
	}

	public Object getValueAt(final int row, final int col) {
		
		Message msg = messages.get(row);
		Object val = null;
		try {
			switch (col) {
			case 0 : val = msg.getJMSMessageID(); break;
			case 1 : val = msg.toString(); break;
			case 2 : val =  new Date(msg.getJMSTimestamp()); break;
			case 3 : val =  msg.getJMSDeliveryMode(); break;
			case 4 : val =  new Date(msg.getJMSExpiration()); break;
			case 5 : val =  msg.getJMSPriority(); break;
			case 6: val =  msg.getJMSCorrelationID(); break;
			default: val =  "Unknown column value";
			// Following give no value or an error when retrieved:
			//case 2 : val =  msg.getJMSType(); break;
			//case 4 : val =  Boolean.toString(msg.getJMSRedelivered()); break;
			//case 7 : val =  msg.getJMSReplyTo().toString(); break;
			};
		} catch (JMSException e) {
			val = "Error";
		}
		return val;
	}
	
	public int getRowCount() {
		return messages.size();
	}

	public Message getSelectedMessage(int index) {

		if (index > -1)
			return messages.get(index);
		return null;
	}
	
	public void refresh() {
		
		if (refreshTask != null && refreshTask.isRunning())
			return;
		refreshTask = new RetrieveMessagesTask();
		Thread t = new Thread(refreshTask);
		t.start();
	}
	
	private void tableUpdated() {
		fireTableDataChanged();
	}
	
	class RetrieveMessagesTask implements Runnable {
		
		private boolean stop;
		private boolean running;
		
		public RetrieveMessagesTask() {
			super();
			stop = false;
			running = true;
		}
		
		public void stopRetrieving() {
			stop = true;
		}
		
		public boolean isRunning() {
			return running;
		}

		public void run() {

			running = true;
			queueWindow.setStatus("Refreshing messages table ...");
			messages = getMessages(mc, qName);
			fireTableDataChanged();
			running = false;
		}

		private List<Message> getMessages(final MarioContext mc, final String queueName) {

			Connection con = null;
			LinkedList<Message> msgs = new LinkedList<Message>();
			try {
				Queue queue = (Queue)mc.ic.lookup("queue/" + queueName);
				ConnectionFactory cf = JmsUtil.getConnectionFactory(mc.ic);
				if (mc.userName != null) {
					con = cf.createConnection(mc.userName, mc.passWord);
				} else {
					con = cf.createConnection();
				}
				Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);

				QueueBrowser qbrowser = session.createBrowser(queue);
				Enumeration<?> qamsgs = qbrowser.getEnumeration();
				int i = 0;
				long t = System.currentTimeMillis();
				while (qamsgs.hasMoreElements() && !stop) {
					msgs.add((Message)qamsgs.nextElement());
					i++;
					if (i % 10 == 0) {
						queueWindow.setStatus("Refreshing messages table, messages retrieved: " + i);
						if (System.currentTimeMillis() - t > 1000) {
							// For some reason the table is not (re-)drawn.
							tableUpdated();
							t = System.currentTimeMillis();
						}
					}
				}
				queueWindow.setStatus("Messages table refreshed, got " + i + " messages.");
				qbrowser.close();
			} catch (Exception e) {
				Log.logError("Could not get messages from queue " + queueName + ": " + Log.exception2String(e));
				msgs = new LinkedList<Message>();
			} finally {
				JmsUtil.closeConnection(con);
			}
			return msgs;
		}
	}
	
	public void dispose() {
		
		if (refreshTask != null)
			refreshTask.stopRetrieving();
		messages.clear();
	}

}
