package nl.intercommit.mario.tools.jbossmon;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Enumeration;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.java.dev.fwutil.KeyAction;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.MFrame;
import net.java.dev.fwutil.Settings;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HBox;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VBox;
import net.java.dev.hvlayout.VLayout;
import nl.intercommit.mario.tools.JmsUtil;
import nl.intercommit.mario.tools.MarioContext;
import nl.intercommit.mario.tools.xml.XmlTextPane;

public class QBrowser extends MFrame implements ActionListener, ListSelectionListener {
	
	private static final long serialVersionUID = -6881929861927739100L;
	private String qname;
	private final JLabel status;
	private final JLabel msgTextStatus;
	private final JButton brefresh;
	private final JButton bsave;
	private final JButton bdelete;
	private final MsgsTable msgsTable; 
	private final JSplitPane viewSplit;
	private final JSplitPane msgSplit;
	private final XmlTextPane tmsgText;
	private final JTextArea tmsgProps;
	
	public QBrowser(final JbServerMon parentWindow, MarioContext mc, final String queueName) {
		
		super(parentWindow, "Browse queue " + queueName);
		if (ICITMon.ICITLogo != null)
			setIconImage(ICITMon.ICITLogo.getImage());
		qname = queueName;
		Container mainPanel = getContentPane();
		mainPanel.setLayout(new VLayout());
		VBox leftPane = new VBox();
		VBox rightPane = new VBox();
		
		HBox hb = new HBox(HBox.CENTER);
		brefresh = FormUI.getButton(this, "Refresh (F5)");
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0), brefresh);
		getRootPane().getActionMap().put(brefresh, new KeyAction(brefresh, this));
		hb.add(brefresh);
		leftPane.add(hb);
		msgsTable = new MsgsTable(mc, this, queueName);
		msgsTable.getSelectionModel().addListSelectionListener(this);
		leftPane.add(HVProps.getScroller(msgsTable, 3 * HVProps.defaultLineWidth, 20, false));
		
		hb = new HBox();
		bsave = FormUI.getButton(this, "Save");
		hb.add(bsave);
		bdelete = FormUI.getButton(this, "Delete");
		HBox hbo = new HBox(HBox.OPPOSITELINEAXIS);
		hbo.add(bdelete);
		hb.add(hbo);
		leftPane.add(hb);
		
		msgTextStatus = FormUI.getLabel("Select a message from the table to show", 2 * HVProps.defaultLineWidth);
		rightPane.add(msgTextStatus);
		
		tmsgProps = new JTextArea();
		JComponent jmsgProps = HVProps.getScroller(tmsgProps, 2 * HVProps.defaultLineWidth, 5, false);
		tmsgText = new XmlTextPane();
		JComponent jmsgText = HVProps.getScroller(tmsgText, 2 * HVProps.defaultLineWidth, 15, false);
		
		msgSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				jmsgProps, jmsgText);
		Object odivLoc = Settings.get("Split location msgs: " + getTitle());
		if (odivLoc != null) {
			int divloc = ((Integer)odivLoc).intValue();
			msgSplit.setDividerLocation(divloc);
		} else {
			msgSplit.setDividerLocation(HVProps.defaultLineHeight * 3);
		}
		
		rightPane.add(msgSplit);

		viewSplit= new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				leftPane, rightPane);
		odivLoc = Settings.get("Split location: " + getTitle());
		if (odivLoc != null) {
			int divloc = ((Integer)odivLoc).intValue();
			viewSplit.setDividerLocation(divloc);
		} else {
			viewSplit.setDividerLocation((int)(HVProps.defaultLineWidth * 2.5));
		}

		mainPanel.add(viewSplit);
		status = FormUI.getLabel("Initializing ...", 2 * HVProps.defaultLineWidth);
		mainPanel.add(status);

		display(parentWindow);
		status.setText("Ready");
		msgsTable.refresh();
	}

	public void setStatus(final String textline) {

		status.setText(textline);
		if (Settings.appDebug)
			Log.log(qname + " Msgs - " + textline);
	}
	
	public void dispose() {

		Settings.put("Split location msgs: " + getTitle(), new Integer(msgSplit.getDividerLocation()));
		Settings.put("Split location: " + getTitle(), new Integer(viewSplit.getDividerLocation()));
		msgsTable.dispose();
		super.dispose();
	}

	public void actionPerformed(ActionEvent ae) {
		
		if (ae.getSource() == brefresh) {
			msgsTable.refresh();
		} else if (ae.getSource() == bsave) {
			msgsTable.saveSelectedMsgs();
		} else if (ae.getSource() == bdelete) {
			msgsTable.deleteSelectedMsgs();
		}
	}

	public String getServer() {
		return ((JbServerMon)parentWindow).getServerURL();
	}
	
	public void showMsgText(String msgText) {
		tmsgText.setText(msgText);
	}

	/** 
	 * Listens for the selected message in the messages table and shows the text.
	 *  
	 */
	public void valueChanged(ListSelectionEvent e) {

		if (e.getValueIsAdjusting())
			return;
       	showMessage(msgsTable.getSelectedMessage());
	}

	@SuppressWarnings("unchecked")
	private void showMessage(Message msg) {
		
		if (msg == null) {
			msgTextStatus.setText("No message selected.");
			tmsgText.setText("");
			tmsgProps.setText("");
			return;
		}
		
		try { msgTextStatus.setText("Properties and contents of " + msg.getJMSMessageID());
		} catch (JMSException e2) {}
		
		Enumeration<String> msgPropsEnum = null;
		try { msgPropsEnum = msg.getPropertyNames();
		} catch (JMSException e1) {	tmsgProps.setText(Log.exception2String(e1)); }
		
		if (msgPropsEnum != null) {
			String propName;
			String propValue;
			StringBuilder s = new StringBuilder();
			while (msgPropsEnum.hasMoreElements()) {
				propName = msgPropsEnum.nextElement();
				s.append(propName + ": ");
				propValue = null;
				try {
					propValue = msg.getObjectProperty(propName).toString();
				} catch (JMSException jmse) {
					propValue = "Error: " + jmse.getLocalizedMessage();
				}
				s.append(propValue + "\n");
			}
			if (s.length() > 0) {
				tmsgProps.setText(s.toString());
			} else {
				tmsgProps.setText("No custom properties available for this message.");
			}
		}

		tmsgText.setText(JmsUtil.getMessageContentsAsString(msg));
		tmsgText.setCaretPosition(0);
	}

}
