package nl.intercommit.mario.tools.jbossmon;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.MFrame;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.UIMessages;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HBox;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VLayout;
import nl.intercommit.mario.tools.ExampleMsgObject;
import nl.intercommit.mario.tools.JmsUtil;
import nl.intercommit.mario.tools.MarioContext;
import nl.intercommit.mario.tools.xml.XmlTextPane;

public class SendMsg extends MFrame implements ActionListener {

	private static final long serialVersionUID = 226657691688489390L;
	private final String qname;
	private final JLabel status;
	private final JButton bstart;
	private final JButton bstop;
	private final JButton bgenMsgText;
	//private JTextField tnumberOfmsgs;
	private final JLabel ltextDescription;
	private final XmlTextPane tmsgText;
	private final JTextField tnumberOfMsgs;
	private final JTextField tsendDelay;
	private final JCheckBox buseTextMsg;
	private final JCheckBox bgenerateId;
	private final SimpleDateFormat simpleDateFormat2;
	private Thread sentMsgsThread = null;
	private SentMsgsTask sentMsgsTask = null;

	public SendMsg(final JbServerMon parentWindow, final String queueName) {

		super(parentWindow, "Send message to " + queueName);
		if (ICITMon.ICITLogo != null)
			setIconImage(ICITMon.ICITLogo.getImage());
		qname = queueName;

		simpleDateFormat2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

		Container mainPanel = getContentPane();
		mainPanel.setLayout(new VLayout());

		HBox hb = new HBox();
		buseTextMsg = FormUI.getCheckBox(this, "Send JBoss text message", HVProps.defaultButtonWidth * 2);
		buseTextMsg.setSelected(true);
		hb.add(buseTextMsg);
		bgenMsgText = FormUI.getWideButton("Generate example text");
		bgenMsgText.addActionListener(this);
		hb.add(bgenMsgText);
		mainPanel.add(hb);

		ltextDescription = FormUI.getLabel("ICIT Example Message XML", 0);
		mainPanel.add(ltextDescription);
		tmsgText = new XmlTextPane();
		tmsgText.setText("Sample text");

		mainPanel.add(HVProps.getScroller(tmsgText, 3 * HVProps.defaultLineWidth, 20, false));

		bgenerateId = FormUI.getCheckBox("Generate new message ID for each message", HVProps.defaultLineWidth);
		bgenerateId.setSelected(true);
		mainPanel.add(bgenerateId);
		
		hb = new HBox();
		tnumberOfMsgs = FormUI.getTextField("1", HVProps.defaultButtonWidth);
		HVProps.doNotGrowWidth(tnumberOfMsgs);
		hb.add(tnumberOfMsgs);
		hb.add(FormUI.getLabel("Number of messages", HVProps.defaultButtonWidth * 2, false, true));
		mainPanel.add(hb);

		hb = new HBox();
		tsendDelay = FormUI.getTextField("1", HVProps.defaultButtonWidth);
		HVProps.doNotGrowWidth(tsendDelay);
		hb.add(tsendDelay);
		hb.add(FormUI.getLabel("Delay between messages (in milliseconds)", HVProps.defaultButtonWidth * 2, false, true));
		mainPanel.add(hb);

		hb = new HBox();
		bstart = FormUI.getButton(this, "Start");
		hb.add(bstart);
		bstop = FormUI.getButton(this, "Stop");
		bstop.setEnabled(false);
		hb.add(bstop);
		mainPanel.add(hb);
		status = FormUI.getLabel("Initializing ...", 2 * HVProps.defaultLineWidth);
		mainPanel.add(status);
		
		display(parentWindow);
		status.setText("Ready");
		tmsgText.requestFocus();
	}

	public void setStatus(final String textline) {

		status.setText(textline);
		// Has no effect:
		//status.paintImmediately(status.getBounds());
		if (Settings.appDebug)
			Log.log(qname + " - " + textline);
	}
	
	public void actionPerformed(final ActionEvent ae) {

		if (ae.getSource() == bgenMsgText) {
			if (buseTextMsg.isSelected()) {
				tmsgText.setText("Sample text for text message generated at " + simpleDateFormat2.format(new Date()));
			} else {
				ExampleMsgObject exo = new ExampleMsgObject();
				exo.generateValues();
				String t = getMsgText(exo);
				if (t != null)
					tmsgText.setText(t);
			}
		} else if (ae.getSource() == bstart) {
			sentMsg();
		} else if (ae.getSource() == bstop) {
			sentMsgsTask.stopSending();
		} else if (ae.getSource() == buseTextMsg) {
			if (buseTextMsg.isSelected()) {
				bgenerateId.setEnabled(false);
				bgenerateId.setSelected(false);
				bgenMsgText.setText("Generate example text");
				ltextDescription.setText("Text to send with JBoss Text Message");
			} else {
				ltextDescription.setText("Example Message XML");
				bgenerateId.setEnabled(true);
				bgenMsgText.setText("Generate Example Message XML");
			}
		} 
	}

	public void dispose() {
		if (sentMsgsTask != null)
			sentMsgsTask.stopSending();
		super.dispose();
	}

	private void sentMsg() {

		int numberOfmsgs = 0;
		int msgDelay = 0;
		try {
			numberOfmsgs = Integer.valueOf(tnumberOfMsgs.getText());
		} catch (NumberFormatException ne) {
			setStatus("Number of messages is invalid, must be a valid number.");
			return;
		}
		if (numberOfmsgs < 1) {
			setStatus("Number of messages must be greater then 0.");
			return;
		}
		try {
			msgDelay = Integer.valueOf(tsendDelay.getText());
		} catch (NumberFormatException ne) {
			setStatus("Send delay is invalid, must be a valid number.");
			return;
		}
		if (msgDelay < 0) {
			setStatus("Send delay must be greater then or equal to 0.");
			return;
		}
		setStatus("Preparing to send messages.");
		bstart.setEnabled(false);
		sentMsgsTask = new SentMsgsTask(numberOfmsgs, msgDelay, buseTextMsg.isSelected(), bgenerateId.isSelected());
		sentMsgsThread = new Thread(sentMsgsTask);
		sentMsgsThread.start();
	}
	
	private class SentMsgsTask implements Runnable {
		
		private int numberOfMsgs;
		private int sleepTime;
		private boolean useTextMsg = false;
		private boolean generateId = false;
		private boolean stop = false;
		private Serializable mcm = null;
		
		public SentMsgsTask(final int numberOfMessages, final int msgDelay, 
				final boolean useTM, final boolean genId) {
			super();
			numberOfMsgs = numberOfMessages;
			sleepTime = msgDelay;
			useTextMsg = useTM;
			generateId = genId;
		}
		
		public void stopSending() {
			stop = true;
		}
		
		public void run() {
			Connection con = null;
			String msgId = null;
			String msgText = tmsgText.getText();
			if (!useTextMsg) {
				mcm = getObjectMessage(msgText);
				if (mcm == null) {
					bstart.setEnabled(true);
					return;
				}
			}
			bstop.setEnabled(true);
			try {
				MarioContext mc = ((JbServerMon)parentWindow).getMonTable().getMonitor().getMC();
				Queue queue = (Queue)mc.ic.lookup("queue/" + qname);
				ConnectionFactory cf = JmsUtil.getConnectionFactory(mc.ic);
				if (mc.userName != null) {
					con = cf.createConnection(mc.userName, mc.passWord);
				} else {
					con = cf.createConnection();
				}
				Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
				MessageProducer sender = session.createProducer(queue);
				int i = 0;
				while (!stop && i < numberOfMsgs) {
					Message msg = null;
					if (useTextMsg) {
						msg = session.createTextMessage(msgText);
					} else {
						if (generateId) {
							((ExampleMsgObject)mcm).setUuid(UUID.randomUUID().toString());
						}
						msg = session.createObjectMessage(mcm);
					} 
					sender.send(msg);
					msgId = msg.getJMSMessageID();
					setStatus("Message " + (i+1) + " of " + numberOfMsgs + " sent, JMS Message ID = " + msgId);
					i++;
					if (sleepTime > 0) {
						try { Thread.sleep(sleepTime); } catch (InterruptedException ie){}
					}
				}
			} catch (Exception e) {
				setStatus("Error during sending of message(s): " + e.getLocalizedMessage());
				Log.logError(e);
			} finally {
				JmsUtil.closeConnection(con);
			}
			bstart.setEnabled(true);
			bstop.setEnabled(false);
		}
	}
	
	private Serializable getObjectMessage(String msgText) {
		
		ByteArrayInputStream xmlin;
		try {
			xmlin = new ByteArrayInputStream(msgText.getBytes("UTF-8"));
		} catch (UnsupportedEncodingException e1) {
			Log.logError(e1);
			UIMessages.showErrorMessage(this, "UTF-8 not supported", "The required encoding UTF-8 for XML is not supported.");
			return null;
		}
		XMLDecoder dec = new XMLDecoder(xmlin);
		Object o = null;
		try {
			o = dec.readObject();
		} catch (Exception e) {
			o = null;
		}
		try {xmlin.close();} catch (IOException unused){} 
		if (o == null || (!(o instanceof Serializable))) {
			UIMessages.showErrorMessage(this, "Not valid", "The XML-text is not valid.");
			return null;
		}
		return (Serializable)o;
	}

	/* Converts a Serializable object to XML-text. */
	private String getMsgText(Serializable mm) {

		ByteArrayOutputStream xmlout = new ByteArrayOutputStream();
		XMLEncoder enc = new XMLEncoder(xmlout);
		enc.writeObject(mm);
		enc.close();
		String text = xmlout.toString();
		try {xmlout.close(); } catch (Exception e) {}
		return text;
	}
}
